#include "GyverMotor.h"
#include <Servo.h>
#include <VL6180X.h>
#include <NewPing.h>

#define SERVO 11
#define ENA 5
#define INA 3
#define INB 4
#define INC 7
#define IND 9
#define ENB 6
#define LINEL A0
#define LINER A1
#define ECHO 8
#define TRIG 10

Servo myservo;
VL6180X sensor;
NewPing wallsensor(TRIG, ECHO, 700); 

const int NUM_READ = 5;

int grayl = (402+24)/2;
int grayr = (476+26)/2;
int linel = 0;
int liner = 0;
unsigned long timer = 0;
unsigned long wall_timer = 0;
int vl = 65;
int vr = 50;
int offset_v = 15;
int dist = 999;
int wall = 1;
bool stopline = false;
int wall_dist = 7;
int around_time = 2000;
int rotate_servo_angle = 160;
int rotate_time = 800;
int resist = 40;

GMotor rmotor(DRIVER3WIRE, INA, INB, ENA, HIGH);
GMotor lmotor(DRIVER3WIRE, INC, IND, ENB, HIGH);

void setup(){
  Serial.begin(9600);
  pinMode(SERVO, OUTPUT);
  pinMode(TRIG, INPUT);
  pinMode(ECHO, INPUT);
  lmotor.setMode(AUTO);
  lmotor.setDirection(REVERSE);
  rmotor.setMode(AUTO);
  myservo.attach(SERVO);
  sensor.init();
  sensor.configureDefault();
  sensor.setScaling(1);
  sensor.setTimeout(500);
  //НАЧАЛО
  myservo.write(160);
  line_to_cube();//движение до куба
  for(int i = 0; i < 3; i++){
    //НАЧАЛО ИТЕРАЦИИ
    rotate_to_cube();//вращение до куба
    corner_around_cube();//объезд куба на 90 градусов
    line_to_stopline();//движение до стоп-линии
    for(int i = 0; i < 5; i++){
      int delta_wall = wallsensor.convert_cm(wallsensor.ping());
      if(delta_wall != 0){
        wall = (delta_wall < wall_dist) ? 0 : 1;
      }
    }
    if(wall == 1){//есть ли банка
      around_time = 2000;
      rotate_servo_angle = 160;
      rotate_time = 800;
      rotate180();//вращение на 180 градусов
      line_to_cube();//движение до куба
      rotate_to_cube();//вращение до куба
      around_cube();//объезд куба на 180 градусов
      line_to_stopline();//движение до стоп-линии
      myservo.write(100);
      rotate180();//вращение на 180 градусов
      //БАНКА ЗАХВАЧЕНА
      line_to_cube();//движение до куба
      vl = 90;
      vr = 75;
      rotate_to_cube();//вращение до куба
      vl = 65;
      vr = 50;
      around_cube();//объезд куба на 180 градусов
      line_to_stopline();//движение до стоп-линии
      myservo.write(100);
      //БАНКА ОТПУЩЕНА
      rotate_servo_angle = 10;
      rotate_time = 300;
      rotate180();//вращение на 180 градусов
      line_to_cube();//движение до куба
      rotate_to_cube();//вращение до куба
      corner_around_cube();//объезд куба на 90 градусов
      //КОНЕЦ ИТЕРАЦИИ
    }
    else{
      around_time = 2000;
      rotate_servo_angle = 160;
      rotate_time = 800;
      myservo.write(100);
      rotate180();//вращение на 180 градусов
      //БАНКА ЗАХВАЧЕНА
      line_to_cube();//движение до куба
      vl = 90;
      vr = 75;
      rotate_to_cube();//вращение до куба
      vl = 60;
      vr = 45;
      around_cube();//объезд куба на 180 градусов
      line_to_stopline();//движение до стоп-линии
      myservo.write(100);
      //БАНКА ОТПУЩЕНА
      rotate_servo_angle = 10;
      rotate_time = 300;
      rotate180();//вращение на 180 градусов
      line_to_cube();//движение до куба
      rotate_to_cube();//вращение до куба
      around_time = 3000;
      around_cube();//объезд куба на 270 градусов
      //КОНЕЦ ИТЕРАЦИИ
    }
    myservo.write(160);//возвращение лапки сервопривода
  }
  corner_around_cube();//объезд куба на 90 градусов
  line_to_stopline();//движение до стоп-линии
  lmotor.setSpeed(vl);
  rmotor.setSpeed(vr);
  delay(300);
  lmotor.setSpeed(0);
  rmotor.setSpeed(0);
  //КОНЕЦ
}

void loop() { 

}

void corner_around_cube(){
  lmotor.setSpeed(vl/2);
  rmotor.setSpeed(vr+30);
  delay(1200);
  while(analogRead(LINER) < resist){
    lmotor.setSpeed(vl-30);
    rmotor.setSpeed(-vr-50);
  }
  lmotor.setSpeed(0);
  rmotor.setSpeed(0);
}

void around_cube(){
  lmotor.setSpeed(vl/2);
  rmotor.setSpeed(vr+15);
  delay(around_time);
  while(analogRead(LINEL) < resist){
    lmotor.setSpeed(vl/2);
    rmotor.setSpeed(vr+15);
  }
  lmotor.setSpeed(vl/2);
  rmotor.setSpeed(vr+15);
  delay(400);
  while(analogRead(LINEL) < resist){
    lmotor.setSpeed(vl-10);
    rmotor.setSpeed(-vr+10);
  }
  lmotor.setSpeed(0);
  rmotor.setSpeed(0);
}

void stop(){
  while(true){
    lmotor.setSpeed(0);
    rmotor.setSpeed(0);  
    Serial.print(analogRead(LINEL));Serial.print(' ');
    Serial.print(analogRead(LINER));Serial.print(' ');
    int delta_wall = wallsensor.convert_cm(wallsensor.ping());
    if(delta_wall != 0){
      wall = delta_wall;
    }
    Serial.print(wall);Serial.print(' ');
    Serial.println(sensor.readRangeSingleMillimeters());
    delay(100);
  }
}

void rotate180(){
  lmotor.setSpeed(-vl-30);
  rmotor.setSpeed(vr+50);
  delay(rotate_time);
  myservo.write(rotate_servo_angle);
  delay(800-rotate_time);
  while(analogRead(LINER) < resist){
    lmotor.setSpeed(-vl);
    rmotor.setSpeed(vr-30);
  }
  lmotor.setSpeed(0);
  rmotor.setSpeed(0);
  delay(1000);
  while(analogRead(LINEL) < resist){
    lmotor.setSpeed(0);
    rmotor.setSpeed(-vr-10);
  }
  lmotor.setSpeed(0);
  rmotor.setSpeed(0);
}

void line_to_stopline(){
  stopline = false;
  while(stopline == false){
    line_move();
  }
  lmotor.setSpeed(0);
  rmotor.setSpeed(0);
}

void line_to_cube(){
  wall = 1;
  stopline = false;
  while(wall == 1){
    line_move();
  }
  lmotor.setSpeed(0);
  rmotor.setSpeed(0);  
}

void rotate_to_cube(){
  dist = 999;
  lmotor.setSpeed(vl);
  rmotor.setSpeed(-vr/3);
  delay(700);
  while(dist > 120){
    lmotor.setSpeed(vl);
    rmotor.setSpeed(0);
    dist = sensor.readRangeSingleMillimeters();
  }
  lmotor.setSpeed(0);
  rmotor.setSpeed(0);  
}

void line_move(){
  if(millis() - wall_timer >= 100){
    int delta_wall = wallsensor.convert_cm(wallsensor.ping());
    if(delta_wall != 0){
      wall = (delta_wall < wall_dist) ? 0 : 1;
    }
    wall_timer = millis();
  }
  if(millis() - timer >= 5){
    linel = analogRead(LINEL);
    liner = analogRead(LINER);
    if(stopline == false){
      if(linel > grayl){
        if(liner > grayr){
          lmotor.setSpeed(0);
          rmotor.setSpeed(0);
          stopline = true;
        }
        else{
          lmotor.setSpeed(-vl/4);
          rmotor.setSpeed(vr);
        }
      }
      else{
        if(liner > grayr){
          lmotor.setSpeed(vl);
          rmotor.setSpeed(-vr/4);
        }
        else{
          lmotor.setSpeed(vl);
          rmotor.setSpeed(vr);
        }
      }
    }
    timer = millis();
  }
}